package net.wanji.web.service.scheme.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import net.wanji.common.enums.LightsAddressEnum;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.dao.mapper.BaseCrossLaneLightsMapper;
import net.wanji.databus.dao.mapper.BaseCrossLightsMapper;
import net.wanji.databus.dao.mapper.BaseCrossPhaseLightsMapper;
import net.wanji.databus.dao.mapper.LaneInfoMapper;
import net.wanji.databus.po.BaseCrossLightsPO;
import net.wanji.databus.po.CrossLaneLightsPO;
import net.wanji.databus.po.LaneInfoPO;
import net.wanji.web.common.enums.CrossLisghtsLaneDirEnum;
import net.wanji.web.common.exception.LightLaneConflictException;
import net.wanji.web.dto.DirListElement;
import net.wanji.web.dto.LaneListElement;
import net.wanji.web.dto.LedConfigListElement;
import net.wanji.web.dto.SaveLaneInfoDTO;
import net.wanji.web.mapper.base.BaseRidInfoMapper;
import net.wanji.web.mapper.scheme.CrossDirInfoMapper;
import net.wanji.web.mapper.scheme.LaneSegmentMapper;
import net.wanji.web.po.base.BaseRidInfo;
import net.wanji.web.po.scheme.CrossDirInfoPO;
import net.wanji.web.service.scheme.CrossConfigService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2022/12/20 10:27
 */
@Service
public class CrossConfigServiceImpl implements CrossConfigService {
    //
    private final CrossDirInfoMapper crossDirInfoMapper;
    private final LaneInfoMapper laneInfoMapper;
    private final BaseCrossLaneLightsMapper baseCrossLaneLightsMapper;
    private final LaneSegmentMapper laneSegmentMapper;
    private final BaseCrossLightsMapper baseCrossLightsMapper;
    private final BaseRidInfoMapper baseRidInfoMapper;

    // 机动车灯数据库类型与前端类型对应关系
    private static final Map<Integer, Integer> MOTOR_VEH_MAP = new HashMap<>(5);
    static {
        MOTOR_VEH_MAP.put(1, 1);
        MOTOR_VEH_MAP.put(2, 3);
        MOTOR_VEH_MAP.put(3, 4);
        MOTOR_VEH_MAP.put(4, 5);
        MOTOR_VEH_MAP.put(5, 6);
    }
    // 非机动车灯数据库类型与前端类型对应关系
    private static final Map<Integer, Integer> NON_MOTOR_VEH_MAP = new HashMap<>(5);
    static {
        MOTOR_VEH_MAP.put(10, 2);
        MOTOR_VEH_MAP.put(11, 3);
        MOTOR_VEH_MAP.put(12, 4);
        MOTOR_VEH_MAP.put(13, 5);
        MOTOR_VEH_MAP.put(14, 6);
    }

    private final BaseCrossPhaseLightsMapper baseCrossPhaseLightsMapper;

    public CrossConfigServiceImpl(CrossDirInfoMapper crossDirInfoMapper, LaneInfoMapper laneInfoMapper,
                                  BaseCrossLaneLightsMapper baseCrossLaneLightsMapper, LaneSegmentMapper laneSegmentMapper,
                                  BaseCrossLightsMapper baseCrossLightsMapper,
                                  BaseCrossPhaseLightsMapper baseCrossPhaseLightsMapper,
                                  BaseRidInfoMapper baseRidInfoMapper) {
        this.crossDirInfoMapper = crossDirInfoMapper;
        this.laneInfoMapper = laneInfoMapper;
        this.baseCrossLaneLightsMapper = baseCrossLaneLightsMapper;
        this.laneSegmentMapper = laneSegmentMapper;
        this.baseCrossLightsMapper = baseCrossLightsMapper;
        this.baseCrossPhaseLightsMapper = baseCrossPhaseLightsMapper;
        this.baseRidInfoMapper = baseRidInfoMapper;
    }

    @Override
    @Transactional
    public void saveLaneInfo(SaveLaneInfoDTO saveLaneInfoDTO) {
        List<LedConfigListElement> ledConfigList = saveLaneInfoDTO.getLedConfigList();
        if (ledConfigList == null || ledConfigList.size() == 0) {
            // 保存渠化数据
            handleLaneData(saveLaneInfoDTO);
        } else {
            try {
                // 前端传过来的所有灯组代码列表
                Set<String> lightCodeSet = new HashSet<>();
                // 保存灯组设置
                handleLightsData(saveLaneInfoDTO, ledConfigList, lightCodeSet);
                // 保存车道配置（灯组-车道关系表）
                handleLaneConfig(saveLaneInfoDTO, lightCodeSet);
                if (lightCodeSet.size() != 0) {
                    throw new RuntimeException("存在未配置的灯组号");
                }
            } catch (Exception e) {
                throw new LightLaneConflictException("灯组设置与车道配置冲突");
            }
        }
    }

    private void handleLaneConfig(SaveLaneInfoDTO saveLaneInfoDTO, Set<String> lightCodeSet) {
        String crossId = saveLaneInfoDTO.getCrossId();
        List<DirListElement> dirList = saveLaneInfoDTO.getDirList();
        List<CrossLaneLightsPO> crossLaneLightsPOList = new ArrayList<>();
        for (DirListElement dirListElement : dirList) {
            List<LaneListElement> laneList = dirListElement.getLaneList();
            Integer dir = dirListElement.getDir();
            if (!CollectionUtils.isEmpty(laneList)) {
                for (LaneListElement laneListElement : laneList) {
                    // 根据路口ID、方向、车道代码获取车道ID
                    String code = laneListElement.getName(); // 车道代码
                    LaneInfoPO laneInfoPO = laneInfoMapper.selectByCrossIdDirCode(crossId, dir, code);
                    String laneId = laneInfoPO.getId();
                    CrossLaneLightsPO crossLaneLightsPO = new CrossLaneLightsPO();
                    crossLaneLightsPO.setLaneId(laneId);
                    crossLaneLightsPO.setCrossId(crossId);
                    // 根据路口ID和灯组代码获取灯组ID
                    String ledNum = laneListElement.getLedNum();
                    if (ledNum != null) {
                        BaseCrossLightsPO baseCrossLightsPO = baseCrossLightsMapper.selectByCrossIdAndLedNum(crossId, ledNum);
                        Integer lightsId = baseCrossLightsPO.getId();
                        crossLaneLightsPO.setLightsId(lightsId);
                        // 用于判断灯组号是否都配置完成
                        lightCodeSet.remove(ledNum);
                    }
                    crossLaneLightsPOList.add(crossLaneLightsPO);
                }
            }
        }
        baseCrossLaneLightsMapper.deleteByCrossId(crossId);
        baseCrossLaneLightsMapper.insertBatch(crossLaneLightsPOList);
    }

    private void handleLightsData(SaveLaneInfoDTO saveLaneInfoDTO, List<LedConfigListElement> ledConfigList,
                                  Set<String> lightCodeSet) {
        // 保存旧的灯组代码-灯组对象映射关系
        Map<String, BaseCrossLightsPO> oldLightsMap = new HashMap<>();
        List<BaseCrossLightsPO> oldPOList = baseCrossLightsMapper.selectByCrossId(saveLaneInfoDTO.getCrossId());
        if (oldPOList != null) {
            for (BaseCrossLightsPO oldPO : oldPOList) {
                oldLightsMap.put(oldPO.getLightsNo(), oldPO);
            }
        }

        String crossId = saveLaneInfoDTO.getCrossId();
        List<BaseCrossLightsPO> baseCrossLightsPOList = new ArrayList<>();
        for (LedConfigListElement ledConfigListElement : ledConfigList) {
            String lightCode = ledConfigListElement.getCode();
            lightCodeSet.add(lightCode);
            BaseCrossLightsPO baseCrossLightsPO = new BaseCrossLightsPO();
            baseCrossLightsPO.setLightsNo(lightCode);
            // 灯组类型
            setType(ledConfigListElement, baseCrossLightsPO);
            // 灯组放行方向
            String address = ledConfigListElement.getAddress();
            Integer addressCode = LightsAddressEnum.getCodeByMsg(address);
            if (addressCode < 10) { // 非行人灯
                // 将放行方向改为灯组方向
                Integer lightsDir = CrossLisghtsLaneDirEnum.getLaneDirByLightsDir(addressCode);
                baseCrossLightsPO.setDir(lightsDir);
            } else { // 行人灯
                String firstDigit = address.toString().substring(0, 1);
                baseCrossLightsPO.setDir(Integer.parseInt(firstDigit));
                // 如果是行人灯，则不需要配置车道
                lightCodeSet.remove(lightCode);
            }
            baseCrossLightsPO.setSort(ledConfigListElement.getOrder());
            baseCrossLightsPO.setCrossId(crossId);
            baseCrossLightsPOList.add(baseCrossLightsPO);
        }
        baseCrossLightsMapper.deleteByCrossId(crossId);
        baseCrossLightsMapper.insertBatch(baseCrossLightsPOList);

        // 对比旧的灯组信息，如没有发生变化，则更新灯组-相位关系表中的灯组ID
        for (BaseCrossLightsPO newPO : baseCrossLightsPOList) {
            String newNumber = newPO.getLightsNo();
            BaseCrossLightsPO oldPO = oldLightsMap.get(newNumber);
            if (oldPO != null) {
                if (checkIsSame(oldPO, newPO)) {
                    baseCrossPhaseLightsMapper.updateLightsId(oldPO.getId(), newPO.getId());
                }
            }
        }
    }

    private boolean checkIsSame(BaseCrossLightsPO oldPO, BaseCrossLightsPO newPO) {
        return (Objects.equals(oldPO.getDir(), newPO.getDir())
                && Objects.equals(oldPO.getType(), newPO.getType()));
    }

    private void setType(LedConfigListElement ledConfigListElement, BaseCrossLightsPO baseCrossLightsPO) {
        Integer type = ledConfigListElement.getType();
        Integer signalType = ledConfigListElement.getSignalType();
        String address = ledConfigListElement.getAddress();
        if (type == 1) { // 机动车灯
            if (signalType == 1) {
                // 圆饼灯
                baseCrossLightsPO.setType(1);
            } else if (signalType == 3) {
                // 左转灯
                baseCrossLightsPO.setType(2);
            } else if (signalType == 4) {
                // 直行灯
                baseCrossLightsPO.setType(3);
            } else if (signalType == 5) {
                // 右转灯
                baseCrossLightsPO.setType(4);
            } else if (signalType == 6) {
                // 掉头灯
                baseCrossLightsPO.setType(5);
            }
        } else if (type == 2) { // 非机动车灯
            if (signalType == 2) {
                // 无转向的非机动车灯
                baseCrossLightsPO.setType(10);
            } else if (signalType == 3) {
                // 左转非机动车灯
                baseCrossLightsPO.setType(11);
            } else if (signalType == 4) {
                // 直行非机动车灯
                baseCrossLightsPO.setType(12);
            } else if (signalType == 5) {
                // 右转非机动车灯
                baseCrossLightsPO.setType(13);
            } else if (signalType == 6) {
                // 掉头非机动车灯
                baseCrossLightsPO.setType(14);
            }
        } else if (type == 3) { // 公交专用
            if (signalType == 7) {
                baseCrossLightsPO.setType(30);
            }
        } else if (type == 4) { // 行人
            Integer addressCode = LightsAddressEnum.getCodeByMsg(address);
            if (addressCode >= 10 && addressCode <= 99) { // 两位数为一次过街
                baseCrossLightsPO.setType(20);
            } else if (addressCode >= 100 && addressCode % 10 == 1) { // 三位数为二次过街，个位是1表示出口
                baseCrossLightsPO.setType(22);
            } else if (addressCode >= 100 && addressCode % 10 == 2) { // 三位数为二次过街，个位是2表示进口
                baseCrossLightsPO.setType(21);
            }
        }
    }

    private void handleLaneData(SaveLaneInfoDTO saveLaneInfoDTO) {
        String crossId = saveLaneInfoDTO.getCrossId();
        List<DirListElement> dirList = saveLaneInfoDTO.getDirList();
        List<CrossDirInfoPO> crossDirInfoPOList = crossDirInfoMapper.selectByCrossId(crossId);
        List<Integer> dirListFromDb= crossDirInfoPOList.stream()
                .map(CrossDirInfoPO::getDirType)
                .collect(Collectors.toList());
        // 删除数据库中有，前端对象中没有的方向
        for (Integer dirFromDb : dirListFromDb) {
            if (!dirList.contains(dirFromDb)) {
                List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectInDirByCrossIdAndDir(crossId, dirFromDb);
                cleanDataBase(crossId, dirFromDb, laneInfoPOList);
            }
        }
        for (DirListElement dirListElement : dirList) {
            Integer dir = dirListElement.getDir();
            List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectInDirByCrossIdAndDir(crossId, dir);
            List<LaneListElement> laneListFromClient = dirListElement.getLaneList();
            List<LaneListElement> laneListFromDb = buildLaneListFromDb(laneInfoPOList);
            // 比较入参数据与已有数据
            if (!laneListFromClient.equals(laneListFromDb) || CollectionUtil.isEmpty(laneListFromClient)) {
                cleanDataBase(crossId, dir, laneInfoPOList);
                // 插入进口信息
                List<LaneInfoPO> laneInfoPOListForInsert = getLaneInfoPOListForInsert(
                        crossId, dir, laneListFromClient);
                if (laneInfoPOListForInsert.size() > 0) {
                    laneInfoMapper.insertBatch(laneInfoPOListForInsert);
                }
                // 插入路口方向
                CrossDirInfoPO crossDirInfoPO = getCrossDirInfoPO(crossId, dirListElement);
                crossDirInfoMapper.insertOne(crossDirInfoPO);
            }
            // 更新是否有行人道
            String crossDirInfoId = getId(crossId, dirListElement);
            crossDirInfoMapper.updateIsPedestrian(dirListElement.getIsPersonCross(), crossDirInfoId);
        }
    }

    private void cleanDataBase(String crossId, Integer dir, List<LaneInfoPO> laneInfoPOList) {
        // 删除车道表数据
        laneInfoMapper.deleteByCrossIdAndDir(crossId, dir);
        // 删除路口方向表数据
        crossDirInfoMapper.deleteInByDirType(crossId, dir);
        List<String> laneIds = getLaneIds(laneInfoPOList);
        if (laneIds.size()!=0){
            // 删除灯组-车道关系表数据
            baseCrossLaneLightsMapper.deleteByLaneIds(laneIds);
            // 删除车道状态表数据
            laneSegmentMapper.deleteByLaneIds(laneIds);
        }
    }

    @Override
    public SaveLaneInfoDTO listLaneInfo(CrossIdBO crossIdBO) {
        String crossId = crossIdBO.getCrossId();
        SaveLaneInfoDTO saveLaneInfoDTO = new SaveLaneInfoDTO();
        saveLaneInfoDTO.setCrossId(crossId);
        // 构造dirList
        buildDirList(crossId, saveLaneInfoDTO);
        // 构造ledConfigList
        buildLedConfigList(crossId, saveLaneInfoDTO);

        return saveLaneInfoDTO;
    }

    private void buildLedConfigList(String crossId, SaveLaneInfoDTO saveLaneInfoDTO) {
        saveLaneInfoDTO.setLedConfigList(new ArrayList<>());
        List<LedConfigListElement> ledConfigList = saveLaneInfoDTO.getLedConfigList();
        // 获取灯组数据
        List<BaseCrossLightsPO> baseCrossLightsPOList = baseCrossLightsMapper.selectByCrossId(crossId);
        // 赋值给ledConfigList
        for (BaseCrossLightsPO baseCrossLightsPO : baseCrossLightsPOList) {
            LedConfigListElement ledConfigListElement = new LedConfigListElement();
            ledConfigListElement.setId(baseCrossLightsPO.getId());
            ledConfigListElement.setOrder(baseCrossLightsPO.getSort());
            ledConfigListElement.setCode(baseCrossLightsPO.getLightsNo());
            // 处理type
            Integer type = baseCrossLightsPO.getType();
            if (type > 0 && type < 10) { // 机动车灯
                ledConfigListElement.setType(1);
                Integer signalType = MOTOR_VEH_MAP.get(type);
                ledConfigListElement.setSignalType(signalType);
            } else if (type >= 10 && type < 20) { // 非机动车灯
                ledConfigListElement.setType(2);
                Integer signalType = NON_MOTOR_VEH_MAP.get(type);
                ledConfigListElement.setSignalType(signalType);
            } else if (type >= 20 && type < 30) { // 行人灯
                ledConfigListElement.setType(4);
                ledConfigListElement.setSignalType(8);
            } else if (type == 30) { // 公交专用灯
                ledConfigListElement.setType(3);
                ledConfigListElement.setSignalType(7);
            }
            // 灯组放行方向  转化为灯组方向
            Integer dir = baseCrossLightsPO.getDir();
            if (type < 20 || type >= 30) { // 非行人灯
                Integer laneDir = CrossLisghtsLaneDirEnum.getLightsDirByLaneDir(dir);
                String address = LightsAddressEnum.getMsgByCode(laneDir);
                ledConfigListElement.setAddress(address);
            } else if (type == 20) { // 一次过街
                String s = "" + dir + 1;
                setLightAddress(ledConfigListElement, s);
            } else if (type == 22) { // 二次过街-出口
                String s = "" + dir + 21;
                setLightAddress(ledConfigListElement, s);
            } else if (type == 21) { // 二次过街-进口
                String s = "" + dir + 22;
                setLightAddress(ledConfigListElement, s);
            }
            ledConfigList.add(ledConfigListElement);
        }
        // 根据灯组序号排序
//        ledConfigList.sort(Comparator.comparing(LedConfigListElement::getOrder));
        Collections.sort(ledConfigList, Comparator.comparingInt(led -> Integer.valueOf(led.getCode())));
    }

    private void setLightAddress(LedConfigListElement ledConfigListElement, String s) {
        int i = Integer.parseInt(s);
        String address = LightsAddressEnum.getMsgByCode(i);
        ledConfigListElement.setAddress(address);
    }

    private void buildDirList(String crossId, SaveLaneInfoDTO saveLaneInfoDTO) {
        saveLaneInfoDTO.setDirList(new ArrayList<>());
        List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectBycrossId(crossId);
        Map<Integer, List<LaneInfoPO>> collect = laneInfoPOList.stream()
                .collect(Collectors.groupingBy(LaneInfoPO::getDir));
        QueryWrapper<BaseRidInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("end_cross_id", crossId);
        List<BaseRidInfo> ridList = baseRidInfoMapper.selectList(queryWrapper);
        Map<Integer, String> roadNameMap = ridList.stream().collect(Collectors.toMap(o->o.getInDir(),o->o.getName(),
                (existing,replacement) -> existing));
        for (Map.Entry<Integer, List<LaneInfoPO>> entry : collect.entrySet()) {
            Integer key = entry.getKey();
            DirListElement dirListElement = new DirListElement();
            dirListElement.setDir(key);
            dirListElement.setRoadName(roadNameMap.get(key));
            // 获取是否有行人道
            CrossDirInfoPO crossDirInfoPO = crossDirInfoMapper.selectByCrossIdAndDirType(crossId, key);
            Integer isPedestrian = 0;
            if (Objects.nonNull(crossDirInfoPO)) {
                isPedestrian = crossDirInfoPO.getIsPedestrian();
            }
            dirListElement.setIsPersonCross(isPedestrian);
            // 构造laneList
            List<LaneInfoPO> value = entry.getValue();
            List<LaneInfoPO> sortedValue = value.stream().
                    sorted(Comparator.comparing(LaneInfoPO::getSort)).
                    collect(Collectors.toList());
            List<LaneListElement> laneListElementList = new ArrayList<>();
            for (LaneInfoPO laneInfoPO : sortedValue) {
                LaneListElement laneListElement = new LaneListElement();
                String laneId = laneInfoPO.getId();
                laneListElement.setId(laneId);
                Integer turn = laneInfoPO.getTurn();
                laneListElement.setDirection(turn);
                laneListElement.setLaneType(laneInfoPO.getCategory());
                laneListElement.setName(laneInfoPO.getCode());
                // 根据laneId获取灯组代码
                String lightsCode = baseCrossLaneLightsMapper.selectLightsCodeByLaneId(laneId);
                if (lightsCode != null) {
                    laneListElement.setLedNum(lightsCode);
                }
                laneListElementList.add(laneListElement);
            }
            dirListElement.setLaneList(laneListElementList);
            // 构造行人过街数据
            List<BaseCrossLightsPO> baseCrossLightsPOList = baseCrossLightsMapper.selectByCrossIdAndDir(crossId, key);
            for (BaseCrossLightsPO baseCrossLightsPO : baseCrossLightsPOList) {
                Integer type = baseCrossLightsPO.getType();
                String lightsNo = baseCrossLightsPO.getLightsNo();
                if (type == 20) { // 一次过街
                    dirListElement.setPersonCrossType("oneCross");
                    dirListElement.setOneCross(lightsNo);
                } else if (type == 22) { // 二次过街出口灯
                    dirListElement.setPersonCrossType("twiceCross");
                    dirListElement.setTwiceCrossOut(lightsNo);
                } else if (type == 21) { // 二次过街进口灯
                    dirListElement.setPersonCrossType("twiceCross");
                    dirListElement.setTwiceCrossIn(lightsNo);
                }
            }
            List<DirListElement> dirList = saveLaneInfoDTO.getDirList();
            dirList.add(dirListElement);
        }
        // 返回没有车道的方向
        List<CrossDirInfoPO> crossDirInfoPOList = crossDirInfoMapper.selectByCrossId(crossId);
        for (CrossDirInfoPO crossDirInfoPO : crossDirInfoPOList) {
            List<DirListElement> dirList = saveLaneInfoDTO.getDirList();
            Integer dirType = crossDirInfoPO.getDirType();
            List<LaneInfoPO> laneInfoPOList1 = laneInfoMapper.selectInDirByCrossIdAndDir(crossId, dirType);
            if (CollectionUtil.isEmpty(laneInfoPOList1)) {
                DirListElement dirListElement = new DirListElement();
                dirListElement.setDir(crossDirInfoPO.getDirType());
                dirListElement.setIsPersonCross(crossDirInfoPO.getIsPedestrian());
                dirList.add(dirListElement);
            }
        }
    }

    private String getId(String crossId, DirListElement dirListElement) {
        Integer dir = dirListElement.getDir();
        String id = crossId + "_" + dir + "_" + 1 + "_" + 0;// todo 暂无主辅路序号，赋值0
        return id;
    }

    private CrossDirInfoPO getCrossDirInfoPO(String crossId, DirListElement dirListElement) {
        CrossDirInfoPO crossDirInfoPO = new CrossDirInfoPO();
        Integer dir = dirListElement.getDir();
        crossDirInfoPO.setId(crossId + "_" + dir + "_" + 1 + "_" + 0); // todo 暂无主辅路序号，赋值0
        crossDirInfoPO.setDirType(dir);
        crossDirInfoPO.setInOutType(1);
        crossDirInfoPO.setCrossId(crossId);
        crossDirInfoPO.setLength(0D); // todo 暂无长度，赋值0
        crossDirInfoPO.setIsPedestrian(dirListElement.getIsPersonCross());
        return crossDirInfoPO;
    }

    private List<LaneInfoPO> getLaneInfoPOListForInsert(
            String crossId, Integer dir, List<LaneListElement> laneListFromClient) {
        List<LaneInfoPO> laneInfoPOListForInsert = new ArrayList<>();
        for (int i = 0; i < laneListFromClient.size(); i++) {
            LaneListElement laneListElement = laneListFromClient.get(i);
            LaneInfoPO laneInfoPO = new LaneInfoPO();
            // 序号
            String sortStr = "1" + (i + 1);
            int sort = Integer.parseInt(sortStr);
            // 上一个路口ID
            String preId = "13N8J0B5P10";
            //String preId = laneInfoMapper.selectPreId(crossId, dir);
            //if (ObjectUtil.isEmpty(preId)) {
            //    throw new RidInfoException("基础路网中无上一路口数据");
            //}
             // 主路；900 进口道渠化序号。信控车道类型都是进口车道
            laneInfoPO.setId(dir + crossId + 0 + 900 + sort);
            laneInfoPO.setCode(laneListElement.getName());
            laneInfoPO.setSort(sort);
            laneInfoPO.setDir(dir);
            Integer direction = laneListElement.getDirection();
            laneInfoPO.setTurn(direction);
            laneInfoPO.setCategory(laneListElement.getLaneType());
            laneInfoPO.setCrossId(crossId);
            laneInfoPOListForInsert.add(laneInfoPO);
        }
        for (LaneListElement laneListElement : laneListFromClient) {

        }
        return laneInfoPOListForInsert;
    }

    private List<String> getLaneIds(List<LaneInfoPO> laneInfoPOList) {
        List<String> laneIds = new ArrayList<>();
        for (LaneInfoPO laneInfoPO : laneInfoPOList) {
            String id = laneInfoPO.getId();
            laneIds.add(id);
        }
        return laneIds;
    }

    private List<LaneListElement> buildLaneListFromDb(List<LaneInfoPO> laneInfoPOList) {
        List<LaneListElement> laneListElements = new ArrayList<>();
        List<LaneInfoPO> collect = laneInfoPOList.stream()
                .sorted(Comparator.comparing(LaneInfoPO::getSort))
                .collect(Collectors.toList());
        for (LaneInfoPO laneInfoPO : collect) {
            LaneListElement laneListElement = new LaneListElement();
            Integer turn = laneInfoPO.getTurn();
            laneListElement.setDirection(turn);
            laneListElement.setName(laneInfoPO.getCode());
            laneListElement.setLaneType(laneInfoPO.getCategory());
            laneListElements.add(laneListElement);
        }
        return laneListElements;
    }
}
